/** argparse is a tool for commandline parsing of shellscripts */

#include <LeoArg.hh>
#include <iostream>
#include <string>
#include <list>

using namespace std;

/// extract the option from string
LeoArg::Option parse_line( const string& line );
inline void strip( std::string& str ); ///< strips a string

typedef std::list<string> string_list;
typedef string_list::iterator string_list_it;

string_list split( const string& line, const string& what, int max = -1 ); ///< splittes a string 

/// extracts the first two string from a string list
inline void get_pair( string_list& sl, string& first, string& second );

inline void toupper( std::string &s ); ///< converts a string to upper case 

typedef enum OType ///< possible option types
  {
    ARGLIST,
    STRING,
    BOOL,
    INT,
    DOUBLE,
    FLAG
  };

/// scans the string and set the otype value
inline OType set_o_type( std::string s ); 


OType o_type;

int main( int argc, char** argv )
{
  LeoArg arg( argc, argv );

  string line;

  while( std::getline( std::cin, line ) )
    {
      o_type = FLAG; // default type is FLAG
      LeoArg::Option o = parse_line( line );
      if( arg.get_option( o ) )
	{
	  switch( o_type )
	    {
	    case ARGLIST: 
	      for( LeoArg::ArgListIt it = o.arglist.begin();
		   it != o.arglist.end(); it++ )
		cout << *it << " ";
	      cout << endl;
	      break;
	    case STRING: cout << o.str << endl; break;
	    case BOOL: cout << o.bool_value << endl; break;
	    case INT: cout << o.int_num << endl; break;
	    case DOUBLE: cout << o.double_num << endl; break;
	    case FLAG: cout << o.flag << endl; break;
	    }
	}
      else
	cout << endl;

      arg.clear_cache();
    }

  return 0;
}

LeoArg::Option parse_line( const string& line )
{
  string_list sl = split( line, ";" );
  LeoArg::Option o("","");

  for( string_list_it it = sl.begin(); 
       it != sl.end(); it++ )
    {
      if( it->empty() )
	continue; // protection of some stupid users

      strip( *it );

      if( it->empty() )
	continue; // protection of some stupid users

      string_list ol = split( *it, ":", 2);

      string what, type;
      get_pair( ol, what, type );

      ol.clear(); // free unused mem

      toupper( what );

      if( what == "LONG" )
	o.long_option = type;
      else if( what == "SHORT" )
	o.short_option = type;
      else if( what == "TYPE" )
	o_type = set_o_type( type );
    }

  if( o.long_option.empty() )
    o.long_option = o.short_option;
  if( o.short_option.empty() )
    o.short_option = o.long_option;

  if( o.long_option.empty() )
    o.long_option = o.short_option = "ERROR";

  return o;
}

inline void strip( string& str )
{
  if( str.empty() )
    return;
  
  string::size_type start = str.find_first_not_of( " \t\n\0" );
  string::size_type end = str.find_last_not_of( " \t\n\0" );
  
  if( start == string::npos || end == string::npos )
    str = "";
  else
    str = str.substr( start, end - start + 1 );
}

string_list split( const std::string& line, const string& what, int max )
{
  string::size_type start, end;
  end = start = 0;
  
  string_list sl;

  string::size_type size = what.size();

  bool start_up = true;
  int act = 0;

  while( true )
    {
      end = line.find( what, start );

      act++;

      if( max > 0 )
	if( act >= max )
	  end = string::npos;

      if( end == string::npos )
	{
	  sl.push_back( line.substr( start ));
	  break;
	}
      
      std::string s = line.substr(start, (end - size) - start + 1);
//      cout << line << " " << start << " " << end << " " << s << endl;
      sl.push_back( s );
     
      start = end + size;
    }

  return sl;
}

inline void get_pair( string_list& sl, string& first, string& second )
{
  string_list_it f;

  f = sl.begin();
  if( f != sl.end() )
    first = *f;
  
  f++;
  if( f != sl.end() )
    second = *f;
  else
    second = first;

  strip( first );
  strip( second );
}

inline void toupper( std::string &s )
{
  for( unsigned int i=0; i< s.size(); i++ )
    s[i] = std::toupper( s[i] );
}

inline OType set_o_type( std::string s )
{
  toupper( s );

  if( s == "ARGLIST" )
    return ARGLIST;
  else if( s == "BOOL" )
    return BOOL;
  else if( s == "INT" )
    return INT;
  else if( s == "DOUBLE" )
    return DOUBLE;
  else if( s == "STRING" )
    return STRING;
  else if( s == "FLAG" )
    return FLAG;
      
  return FLAG; // defaults otype is FLAG
}
